import os
import json

from . import autostart
from . import config
from . import depsolver
from . import podman
from . import repo

class App:
    def __init__(self, app_name):
        self.app_name = app_name
        self.env_file = os.path.join(config.DATA_DIR, f'{app_name}.env')

    def get_definition(self, from_file=None):
        if from_file:
            with open(from_file, encoding='utf-8') as f:
                return json.load(f)
        return repo.get_apps()[self.app_name]

    def install(self, is_update=False, from_file=None):
        definition = self.get_definition(from_file)
        version = definition['version']
        containers = definition['containers']

        # Create volumes
        volumes = set()
        for container in containers.values():
            volumes |= set(container.get('volumes', {}))
        existing_volumes = self.get_existing_volumes()
        if is_update:
            # Remove volumes no longer referenced by the containers
            volumes_to_remove = existing_volumes - volumes
            volumes -= existing_volumes
        else:
            # If this is a clean install, remove all volumes with the app label
            volumes_to_remove = existing_volumes
        self.remove_volumes(volumes_to_remove)
        self.create_volumes(volumes)

        # Create env file
        envs = definition.get('environment', {})
        if is_update:
            # Keep old values on update
            for key,value in self.read_env_vars().items():
                if key in envs:
                    envs[key] = value
        self.write_env_vars(envs)

        # Create pod and containers
        self.create_pod(version)
        self.create_containers(containers)

    def update(self, from_file=None):
        self.install(is_update=True, from_file=from_file)

    def uninstall(self):
        autostart.set_app(self.app_name, False)
        self.remove_pod()
        self.remove_env_vars()
        self.remove_volumes(self.get_existing_volumes())

    def create_pod(self, version):
        podman.remove_pod(self.app_name)
        podman.create_pod(self.app_name, version)

    def remove_pod(self):
        podman.remove_pod(self.app_name)

    def read_env_vars(self):
        env_vars = {}
        try:
            with open(self.env_file, encoding='utf-8') as f:
                lines = f.read().splitlines()
            for line in lines:
                key,value = line.split('=', 1)
                env_vars[key] = value
        except FileNotFoundError:
            pass
        return env_vars

    def write_env_vars(self, env_vars):
        os.makedirs(config.DATA_DIR, exist_ok=True)
        with open(self.env_file, 'w', encoding='utf-8') as f:
            for key,value in env_vars.items():
                f.write(f'{key}={value}\n')

    def remove_env_vars(self):
        try:
            os.unlink(self.env_file)
        except FileNotFoundError:
            pass

    def get_existing_volumes(self):
        existing_volumes = podman.get_volumes_for_app(self.app_name)
        strip_len = len(self.app_name)+1
        return set(volume[strip_len:] for volume in existing_volumes)

    def create_volumes(self, volumes):
        for volume in volumes:
            self.create_volume(volume)

    def remove_volumes(self, volumes):
        for volume in volumes:
            self.remove_volume(volume)

    def create_volume(self, volume):
        volume = f'{self.app_name}-{volume}'
        podman.create_volume(self.app_name, volume)

    def remove_volume(self, volume):
        volume = f'{self.app_name}-{volume}'
        podman.remove_volume(volume)

    def create_containers(self, containers):
        deps = depsolver.DepSolver()
        for name,definition in containers.items():
            deps.add(name, definition.get('requires', []))
        container_order = deps.solve()

        hosts = set(containers)
        for name in container_order:
            self.create_container(name, containers[name], hosts)

    def create_container(self, name, definition, hosts):
        name = f'{self.app_name}-{name}'
        image = definition['image']
        volumes = {f'{self.app_name}-{volume}':mount
                   for volume,mount in definition.get('volumes', {}).items()}
        requires = set(f'{self.app_name}-{require}' for require in definition.get('requires', []))
        podman.create_container(self.app_name, name, image, env_file=self.env_file,
                                volumes=volumes, requires=requires, hosts=hosts)

def install(app_name, from_file=None):
    App(app_name).install(from_file=from_file)

def update(app_name, from_file=None):
    App(app_name).update(from_file=from_file)

def uninstall(app_name):
    App(app_name).uninstall()
