class CircularDependencyError(Exception):
    # Dependecy solver has found a circular dependency between items
    def __init__(self, deps):
        super().__init__(deps)
        self.deps = deps

    def __str__(self):
        result = ['Dependency resolution failed due to circular dependency.',
                  'Unresolved dependencies:']
        result.extend(f'  {item} => {item_deps}' for item, item_deps in self.deps.items())
        return '\n'.join(result)

class MissingDependencyError(Exception):
    # Dependecy solver has found an item that depends on a nonexistent item
    def __init__(self, deps, missing):
        super().__init__(deps, missing)
        self.deps = deps
        self.missing = missing

    def __str__(self):
        result = ['Dependency resolution failed due to missing dependency.',
                  'Missing dependencies:']
        result.append(f'  {self.missing}')
        result.append('Unresolved dependencies:')
        result.extend(f'  {item} => {item_deps}' for item, item_deps in self.deps.items())
        return '\n'.join(result)

class DepSolver:
    def __init__(self):
        self.unresolved = {}

    def add(self, item, dependencies):
        self.unresolved[item] = set(dependencies)

    def solve(self):
        # Returns a list of instances ordered by dependency
        resolved = []
        while self.unresolved:
            # Get a batch of items not depending on anything
            # or originally depending on already resolved items
            batch = {item for item,deps in self.unresolved.items() if not deps}
            if not batch:
                # If there are no such items, check if a dependecy is missing
                wanted_deps = set(dep for deps in self.unresolved.values() for dep in deps)
                missing_deps = wanted_deps - set(self.unresolved)
                if missing_deps:
                    raise MissingDependencyError(self.unresolved, missing_deps)
                # If all dependencies exist, we have found a circular dependency
                raise CircularDependencyError(self.unresolved)
            # Add resolved items to the result and remove from the unresolved ones
            for item in batch:
                resolved.append(item)
                del self.unresolved[item]
            # Remove resolved items from the dependencies of yet unresolved items
            for item in self.unresolved:
                self.unresolved[item] -= batch
        return resolved
